@@ -20,6 +20,10 @@ use core::ops::{Deref, DerefMut};
 #[cfg(feature = "alloc")]
 use alloc::vec::Vec;
 
+/// Re-export of `alloc::borrow::Cow`.
+#[cfg(feature = "alloc")]
+pub use alloc::borrow::Cow;
+
 /// Newtype for `[u8]`.
 ///
 /// Used to implement `Encode` and `Decode` which translate to
@@ -252,12 +256,20 @@ impl Encode for ByteVec {
 #[cfg(any(feature = "derive", feature = "partial-derive-support"))]
 pub trait EncodeBytes {
     fn encode_bytes<W: Write>(&self, e: &mut Encoder<W>) -> Result<(), encode::Error<W::Error>>;
+
+    fn is_nil(&self) -> bool {
+        false
+    }
 }
 
 /// Like [`Decode`] but specific for decoding from byte slices.
 #[cfg(any(feature = "derive", feature = "partial-derive-support"))]
 pub trait DecodeBytes<'b>: Sized {
     fn decode_bytes(d: &mut Decoder<'b>) -> Result<Self, decode::Error>;
+
+    fn nil() -> Option<Self> {
+        None
+    }
 }
 
 #[cfg(any(feature = "derive", feature = "partial-derive-support"))]
@@ -360,6 +372,10 @@ impl<T: EncodeBytes> EncodeBytes for Option<T> {
             e.null()?.ok()
         }
     }
+
+    fn is_nil(&self) -> bool {
+        self.is_none()
+    }
 }
 
 #[cfg(any(feature = "derive", feature = "partial-derive-support"))]
@@ -371,6 +387,10 @@ impl<'b, T: DecodeBytes<'b>> DecodeBytes<'b> for Option<T> {
         }
         T::decode_bytes(d).map(Some)
     }
+
+    fn nil() -> Option<Self> {
+        Some(None)
+    }
 }
 
 /// Freestanding function calling `DecodeBytes::decode_bytes`.
@@ -385,6 +405,14 @@ where
     T::decode_bytes(d)
 }
 
+#[cfg(any(feature = "derive", feature = "partial-derive-support"))]
+pub fn nil<'b, T>() -> Option<T>
+where
+    T: DecodeBytes<'b>
+{
+    T::nil()
+}
+
 /// Freestanding function calling `EncodeBytes::encode_bytes`.
 ///
 /// For use in `#[cbor(with = "minicbor::bytes")]` or `#[cbor(encode_with =
@@ -398,3 +426,10 @@ where
     T::encode_bytes(xs, e)
 }
 
+#[cfg(any(feature = "derive", feature = "partial-derive-support"))]
+pub fn is_nil<T>(xs: &T) -> bool
+where
+    T: EncodeBytes
+{
+    T::is_nil(xs)
+}
